/*
 * dstemp.c
 *
 * Created: 2012-03-15 08:12:33
 *  Author: dsosnowski
 */ 

#include <avr/io.h>
#include "dstemp.h"

/*
#define _1W_PORT	PORTC
#define _1W_DDR		DDRC
#define _1W_IN		PINC
#define _1W_PIN		6
*/

#define TDDR	DDRC
#define TPORT	PORTC
#define TPIN	PINC
#define DSPIN	6

#define DSWIRE_OK		0
#define DSWIRE_ERR1		1		//no reported, probably no DS
#define DSWIRE_ERR2		2		//probably short to GND

#define TYPE_ERROR		0
#define	TYPE_18B20		0x28
#define	TYPE_18S20		0x10

#define RESULT_READY	0
#define WORKING			1
#define ERROR			2



struct {
	char status;
	char type;
	unsigned int data;
	unsigned int temperature;
}ds;


//-------------------------------------------------------
//return raw data from ds
unsigned int DSGetData (void)
{
	//---get coded value--------------
	return ds.data;
}

//-------------------------------------------------------
//return temperature value
unsigned int GetTemperature (void)
{
	return ds.temperature;
}

//-------------------------------------------------------
//simple delay
void DSdelay(char us)
{
  do
  {
	//for 4MHz
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
    //asm("nop");
  } while(--us != 0);
}

//----------------------------------
//1wire reset; 0 - ok, 1 - trouble 
char DSreset(void)
{
	char temp;
	ds.status = DSWIRE_ERR1;
			
	TPORT &= ~(1<<DSPIN);
    TDDR |= (1<<DSPIN);		//set low driver			
	DSdelay(250);			//wait 500us
	DSdelay(250);
	DSdelay(100);
	
	TDDR &= ~(1<<DSPIN);	//disable low drivel
	TPORT &= ~(1<<DSPIN);	
	DSdelay(80);			//wait 70us
	
	temp = TPIN;		
	if (temp & (1<<DSPIN)) return DSWIRE_ERR1; 	//check status 0 - ok, 1 - err 
	
	DSdelay(250);			
	DSdelay(250);			//wait 500us
	DSdelay(100);
	temp = TPIN;		
	if (!(temp & (1<<DSPIN))) return DSWIRE_ERR2; 	//check status 1 - ok, 0 - err 
	
	ds.status = DSWIRE_OK;
	return ds.status;
}

//------------------------------------------
//bit write
void DSwriteb(char bit)
{
	TPORT &= ~(1<<DSPIN);
    TDDR |= (1<<DSPIN);		//set low driver			
	DSdelay(5);				//wait10us
	
	if (bit) TPORT |= (1<<DSPIN);	//if bit == 1 then onew quick to H
	DSdelay(65);
 
	TDDR &= ~(1<<DSPIN);	//at the end set to H
	TPORT &= ~(1<<DSPIN);	
	DSdelay(5);
}

//------------------------------------
//Byte write
void DSwriteB(char byte)
{
	char i;
	for (i=0; i<8; i++)
	{
		DSwriteb(byte & 0x01);
		byte >>= 1;
	}	
}

//--------------------------------
//bits read 0 = 0 , 1...255 = 1
char DSreadb(void)
{
	char bits = 0;
	
	TPORT &= ~(1<<DSPIN);
    TDDR |= (1<<DSPIN);		//set low driver			

	DSdelay(5);				//wait
	TDDR &= ~(1<<DSPIN);	//at the end set to H
	
	DSdelay(8);
	if (TPIN & (1<<DSPIN)) bits = 1;	//read onew state
	
	DSdelay(57);
	return bits;
}

//---------------------------------
//Bytes read
char DSreadB(void)
{
	char i;
	char temp = 0;
	
	for (i=0; i<8; i++)
	{
		temp >>= 1;
		if (DSreadb()) temp |= 0x80; 
	}
	return temp;
}	

//-------------------------------------
//bit 0 - DS18S20; bit 1 - DS18B20
char DStype(void)
{
	if ((DSreset()) == DSWIRE_OK)
	{
		DSwriteB(0x33);				//read ROM
		ds.type = DSreadB();
		if ((ds.type == TYPE_18B20) || (ds.type == TYPE_18S20)) 
			return ds.type;
		//else ds.type = TYPE_ERROR
	}
	ds.type = TYPE_ERROR; 
	return ds.type;
}

//------------------------------------
//must be execute about 1/1s, return = 1 then result ready
//char DSprocess(void)
char DSprocess(char step)
{
	char status;
	//static char step = 0;
	
	if (step == 0)
	{
		DStype();
		status = DSreset();
		if (status == DSWIRE_OK)
		{
			DSwriteB(0xCC);		//skip ROM
			DSwriteB(0x44);		//convert
			//power for convert
			TPORT |= (1<<DSPIN);
			TDDR |= (1<<DSPIN);	
			//step = 1;						
		}
		else return ERROR;
	}
	else
	{
		status = DSreset();
		if (status == DSWIRE_OK)
		{
			DSwriteB(0xCC);		//skip ROM
			DSwriteB(0xBE);		//read		
			ds.data = DSreadB();
			status = DSreadB();
			ds.data += (status<<8); 		
			//step = 0;
		
			//for convert to human form
			TempProcess();

			return RESULT_READY;			
		}		
		//step = 0;			
	}
	return WORKING;
}

//-------------------------------------
//result: 0x8000 = minus bit, 1234 = 123.4 'C
unsigned int TempProcess(void)
{
	unsigned int buffer = 0;
	char val;
	char decimal = 0;
	char minus = 0;
	
	//---get coded value--------------
	buffer = ds.data;

	if (ds.type)
	{
		//---check minus------------------
		if (buffer >= 0x7FF)
		{
			buffer = ~(buffer);
			buffer += 1;
			minus = 1;
		}

		//---check type-------------------	
		if (ds.type == TYPE_18B20)
		{
			val = (0xFF & (buffer>>4));
			decimal = (0x0F & buffer);
			decimal = decimal * 6;
			decimal = (decimal / 10);	
		}	
		
		if (ds.type == TYPE_18S20)
		{
			val = (0xFF & (buffer>>1));	
			decimal = (0x01 & buffer);	
			decimal = (decimal * 5);
		}
	
		buffer = val * 10;
		buffer = buffer + decimal;
		if (minus) buffer = buffer + 0x8000;
	}
	else buffer = 0xFFFF;
	ds.temperature = buffer;
	return ds.temperature;
}






