/*
 * func.c
 *
 * Created: 2012-08-07 09:02:45
 *  Author: dsosnowski
 */ 

#include "defs.h"
#include "characters.h"
#include "dstemp.h"
#include <inttypes.h>
#include "ee_mem.h"

void display_fill(char v1, char v2, char v3, char v4, char v5, char v6);


//----------------------------------
extern const char digits[]; 
extern struct{
	volatile uint32_t counter;	
	volatile char hold;
	volatile char correct;
	volatile uint32_t hold_counter;	
}master;

extern struct{
	volatile uint32_t counter;
	volatile uint32_t hold_counter;	
	volatile char alarm;		
	volatile char state;	
	volatile char hold;
	volatile char pointer1;
	volatile char pointer2;
	volatile char correction1;
	volatile char correction2;
}counter;

struct{
	char msec;
	char sec;
	char min;
	char hour;
}time;

struct{

	char settings;				
	char temp;					//to eeprom
	unsigned int interval;
}clock;

struct{
	uint32_t counter;
	
}stopwatch;

struct{
	uint32_t counter;
	uint32_t result1;			
	uint32_t result2;			
	uint32_t result3;			
	uint32_t result4;			
	uint32_t result5;			
	char settings;
	char result;
	char hold;
	unsigned int interval;
	char dominant;	
}multiwatch;

struct{
	char count1;
	char count2;
	char count3;
}pointer;

struct{
	uint32_t counter;
	uint32_t terminate;			//eeprom
	char settings;
	char mode;
	char alarm;
}score;

struct{
	uint32_t counter;
	uint32_t terminate;			//eeprom
	char settings;
	char alarm;
}down;


static char blink;

//--------------------------------
void Restore(void)
{
	clock.temp = ee_read(CLOCK_EEADR);

	ee_32read(SCORE_EEADR, (char*) &score.terminate);
	if (score.terminate > MSEC_ON_99HOUR) score.terminate = 0; 
	
	ee_32read(DOWN_EEADR, (char*) &down.terminate);
	if (down.terminate > MSEC_ON_99HOUR) down.terminate = 0; 
}

//------------------------------
void Clock(char press)
{
	uint32_t long_temp;

	char digit1 = cNULL;
	char digit2 = cNULL;
	char digit3 = cNULL;
	char digit4 = cNULL;
	char digit5 = cNULL;
	char digit6 = cNULL;

	uint32_t tval;
	char minus = 0;
	char digit;
	char blank;
 	

	
	//---get clock-----------------------------------
	master.hold = MASTER_CATCH;
	while (master.hold);

	//---check user reaction-------------------------
	if (press)
	{
		if (press == PRESSED1) clock.settings++;
		if (clock.settings > 3) clock.settings = 0; 
		
		//change some digits 
		if (clock.settings)
		{
			long_temp = 0;
			switch (clock.settings)
			{
				case 1: long_temp = MSEC_ON_HOUR; 		break;
				case 2: long_temp = MSEC_ON_MIN; 		break;
				case 3: long_temp = MSEC_ON_SEC; 		break;
			}
			switch (press)
			{
				case PRESSED2:
					if (master.hold_counter >= long_temp)
					master.hold_counter -= long_temp;
					master.hold = 2;
				break;
				case PRESSED3:
					master.hold_counter += long_temp;
					master.hold = 2;
				break;
			}
		}
		//other action
		switch (press)
		{
			case PRESSED4: 
				//for future
			break;			
			case PRESSED5:
				//temp on/off
				clock.temp++;
				ee_write(CLOCK_EEADR, clock.temp);
				clock.interval = TEMP_INTERVAL;
			break;				
			case PRESSED6:
				//for future
			break;				
		}
		blink = 0;
		press = 0;
	}
	
	time.hour = (master.hold_counter/ MSEC_ON_HOUR);
	long_temp = (master.hold_counter % MSEC_ON_HOUR);
	time.min =  (long_temp / MSEC_ON_MIN);
	long_temp = (long_temp % MSEC_ON_MIN);
	time.sec =  (long_temp / MSEC_ON_SEC);
	long_temp = (long_temp % MSEC_ON_SEC);
	time.msec = (long_temp & 0xFF);
	
	if (time.hour >= 24)
	{
		time.hour-= 24;
		master.correct = ENABLE;
	}
	
	digit6 = digits[(time.hour/10)];
	if (digit6 == d0) digit6 = cNULL;
	digit5 = digits[(time.hour%10)];
	digit4 = digits[(time.min/10)];
	digit3 = digits[(time.min%10)];
	digit2 = digits[(time.sec/10)];
	digit1 = digits[(time.sec%10)];
		
	//---dots------------------------------------
	if (clock.settings)
	{
		blink++;
		if (blink > BLINK_INTERVAL) blink = 0;
		if (blink > BLINK_PERIOD)
		{
			switch (clock.settings)
			{
				case 1: digit6 = cNULL; digit5 = cNULL; break;
				case 2: digit4 = cNULL; digit3 = cNULL; break;
				case 3: digit2 = cNULL; digit1 = cNULL; break;
			}
		}
		digit5 |= cDOT;
		digit3 |= cDOT;
	}
	else
	{
		if (time.msec < 50)
		{
			digit5 |= cDOT;
			digit3 |= cDOT;		
		}	
	}
	
	//---temp display-----------------------------
	if (clock.temp & 1)
	{
		if (clock.interval < DISP_INTERVAL) clock.interval++;
		else clock.interval = 0;

		if (clock.interval == TEMP_INTERVAL_PROC) DSprocess(0);			
		if (clock.interval == TEMP_INTERVAL)	  DSprocess(1);
		if (clock.interval > TEMP_INTERVAL)
		{
			tval = GetTemperature();
			
			//---correction-----------------
			if (tval & 0x8000) minus = 1;		
			tval = tval & 0x7FF;
			if (tval > 1250) 
			{
				digit6 = cNULL;
				digit5 = cMINUS;
				digit4 = (cMINUS | cDOT);
				digit3 = cMINUS;
			}
			else
			{
				//--convert----------------------
				blank = SET;
				digit = tval / 1000;
				if (digit)
				{
					digit6 = digits[digit];
					blank = CLEAR;
				}
				else
				{
					if (minus) digit6 = cMINUS;
					else	   digit6 = cNULL;	
				}	

				//---------------
				tval = tval % 1000;
				digit = tval / 100;	
				if (digit)
				{
					digit5 = digits[digit];
				}
				else
				{
					if (blank) 
						digit5 = cNULL;
					else
						digit5 = digits[digit];
				}

				//---------------
				tval = tval % 100;
				digit = tval / 10;
				digit4 = (digits[digit] | cDOT);

				//---------------
				tval = tval % 10;
				digit = tval;
				digit3 = digits[digit];
			} 	
			digit2 = cDEG; digit1 = cC;		// *C
		}	
	}

	//---display data--------------------------------
	display_fill(digit6, digit5, digit4, digit3, digit2, digit1);	
}

//------------------------------------------------------------
void StopWatch(char press)
{
	uint32_t long_temp;

	char digit1 = cNULL;
	char digit2 = cNULL;
	char digit3 = cNULL;
	char digit4 = cNULL;
	char digit5 = cNULL;
	char digit6 = cNULL;
	char dots;
	
	//---get score-----------------------------------
	counter.hold = COUNTER_CATCH;
	while (counter.hold);

	multiwatch.dominant = 0;

	stopwatch.counter = counter.hold_counter; 
	if (stopwatch.counter > MSEC_ON_99HOUR) stopwatch.counter = MSEC_ON_99HOUR;

	//---check user reaction-------------------------
	if (press)
	{
			//other action
			switch (press)
			{
				case PRESSED3:
					counter.state = COUNTER_RUN;
					counter.alarm = ALARM_PERIOD;
				break;				
				case PRESSED4:
					counter.state = COUNTER_STOP;			//manual stop
					stopwatch.counter = 0;						//zero display
					counter.counter = 0;
				break;				
				case PRESSED5:
					counter.state = COUNTER_STOP;			//manual stop
				break;				
				case PRESSED6: 
				break;			
			}
		press = 0;
	}

	dots = 0;
	long_temp = stopwatch.counter;
	if (long_temp >= MSEC_ON_HOUR)
	{
		digit1 = (long_temp / MSEC_ON_10HOUR);
		long_temp %= MSEC_ON_10HOUR;
		digit2 = (long_temp / MSEC_ON_HOUR);
		long_temp %= MSEC_ON_HOUR;
		digit3 = (long_temp / MSEC_ON_10MIN);
		long_temp %= MSEC_ON_10MIN;
		digit4 = (long_temp / MSEC_ON_MIN);
		long_temp %= MSEC_ON_MIN;
		digit5 = (long_temp / MSEC_ON_10SEC);
		long_temp %= MSEC_ON_10SEC;
		digit6 = (long_temp / MSEC_ON_SEC);
		long_temp %= MSEC_ON_SEC;
		if (long_temp < 50) dots = 3;
		dots |= 4;		
	}
	else
	{
		long_temp %= MSEC_ON_HOUR;
		digit1 = (long_temp / MSEC_ON_10MIN);
		long_temp %= MSEC_ON_10MIN;
		digit2 = (long_temp / MSEC_ON_MIN);
		long_temp %= MSEC_ON_MIN;
		digit3 = (long_temp / MSEC_ON_10SEC);
		long_temp %= MSEC_ON_10SEC;
		digit4 = (long_temp / MSEC_ON_SEC);	
		long_temp %= MSEC_ON_SEC;		
		digit5 = (long_temp / 10);
		long_temp %= 10;
		digit6 = (long_temp & 0xFF);
		if (digit5 < 5) 	dots |= 1;
		if (!(digit5 & 1))  dots |= 2;
	}

	//---get symbols---------------------------------
	if (digit1) 
		digit1 = digits[digit1];
	else 
		digit1 = cNULL;
	digit2 = digits[digit2];
	digit3 = digits[digit3];
	digit4 = digits[digit4];
	digit5 = digits[digit5];
	digit6 = digits[digit6];

	//---dots rules---------------------------------
		if (counter.state == COUNTER_STOP)
		{
			digit2 |= cDOT;
			digit4 |= cDOT;
			if (dots & 4) digit6 |= cDOT;
		}
		else
		{
			if (dots & 1) digit2 |= cDOT;
			if (dots & 2) digit4 |= cDOT;
			if (dots & 4) digit6 |= cDOT;
		}

	//---display data--------------------------------
	display_fill(digit1, digit2, digit3, digit4, digit5, digit6);
}


//------------------------------------------------------------
void MultiWatch(char press)
{
	uint32_t long_temp;

	char digit1 = cNULL;
	char digit2 = cNULL;
	char digit3 = cNULL;
	char digit4 = cNULL;
	char digit5 = cNULL;
	char digit6 = cNULL;
	char dots;

	
	//---get score-----------------------------------
	counter.hold = COUNTER_CATCH;
	while (counter.hold);
	multiwatch.dominant = 1;
	
	//if (multiwatch.result) multiwatch.counter = counter.hold_counter; 
	multiwatch.counter = counter.hold_counter; 
	

	//---check user reaction-------------------------
	if (press)
	{
			//other action
			switch (press)
			{
				case PRESSED1:
					if (multiwatch.hold < 5) multiwatch.hold++;
					else multiwatch.hold = 1;
					multiwatch.interval = TEMP_SHORTLY_INTERVAL;
				break;
				case PRESSED2:
					if (multiwatch.hold > 1) multiwatch.hold--;
					else multiwatch.hold = 5;
					multiwatch.interval = TEMP_SHORTLY_INTERVAL;
				break;
				case PRESSED3:
					multiwatch.result  = 5;
					counter.state = COUNTER_RUN;
					counter.alarm = ALARM_PERIOD;
				break;				
				case PRESSED4:
					counter.state = COUNTER_STOP;			//manual stop
					multiwatch.hold = 0; 
					counter.counter = 0;
					multiwatch.counter = 0;						//zero display
					multiwatch.result1 = 0;
					multiwatch.result2 = 0;
					multiwatch.result3 = 0;
					multiwatch.result4 = 0;
					multiwatch.result5 = 0;
					//multiwatch.result  = 5;
					multiwatch.result  = 0;
				break;				
				case PRESSED5:
					counter.state = COUNTER_STOP;			//manual stop
				break;				
				case PRESSED6:
					multiwatch.hold = 0; 
				break;			
			}
		press = 0;
	}

	long_temp = multiwatch.counter;
	//---switch dispaly value----------------------------------------
	switch (multiwatch.hold)
	{
		//case 0: long_temp = multiwatch.counter; break;
		case 1: long_temp = multiwatch.result1; break;
		case 2: long_temp = multiwatch.result2; break;
		case 3: long_temp = multiwatch.result3; break;
		case 4: long_temp = multiwatch.result4;	break;
		case 5: long_temp = multiwatch.result5;	break;
	}

	if (long_temp > MSEC_ON_99HOUR) long_temp = MSEC_ON_99HOUR;

	//---calculate------------------------------------------------------
	dots = 0;
	if (long_temp >= MSEC_ON_HOUR)
	{
		digit1 = (long_temp / MSEC_ON_10HOUR);
		long_temp %= MSEC_ON_10HOUR;
		digit2 = (long_temp / MSEC_ON_HOUR);
		long_temp %= MSEC_ON_HOUR;
		digit3 = (long_temp / MSEC_ON_10MIN);
		long_temp %= MSEC_ON_10MIN;
		digit4 = (long_temp / MSEC_ON_MIN);
		long_temp %= MSEC_ON_MIN;
		digit5 = (long_temp / MSEC_ON_10SEC);
		long_temp %= MSEC_ON_10SEC;
		digit6 = (long_temp / MSEC_ON_SEC);
		long_temp %= MSEC_ON_SEC;
		if (long_temp < 50) dots = 3;
		dots |= 4;		
	}
	else
	{
		long_temp %= MSEC_ON_HOUR;
		digit1 = (long_temp / MSEC_ON_10MIN);
		long_temp %= MSEC_ON_10MIN;
		digit2 = (long_temp / MSEC_ON_MIN);
		long_temp %= MSEC_ON_MIN;
		digit3 = (long_temp / MSEC_ON_10SEC);
		long_temp %= MSEC_ON_10SEC;
		digit4 = (long_temp / MSEC_ON_SEC);	
		long_temp %= MSEC_ON_SEC;		
		digit5 = (long_temp / 10);
		long_temp %= 10;
		digit6 = (long_temp & 0xFF);
		if (digit5 < 5) 	dots |= 1;
		if (!(digit5 & 1))  dots |= 2;
	}

	//---get symbols---------------------------------
	if (digit1) 
		digit1 = digits[digit1];
	else 
		digit1 = cNULL;
	digit2 = digits[digit2];
	digit3 = digits[digit3];
	digit4 = digits[digit4];
	digit5 = digits[digit5];
	digit6 = digits[digit6];

	//---dots rules---------------------------------
		if (counter.state == COUNTER_STOP)
		{
			digit2 |= cDOT;
			digit4 |= cDOT;
			if (dots & 4) digit6 |= cDOT;
		}
		else
		{
			if (dots & 1) digit2 |= cDOT;
			if (dots & 2) digit4 |= cDOT;
			if (dots & 4) digit6 |= cDOT;
//			if (!(multiwatch.result)) digit1 |= cDOT;
		}

	//---dispaly result info---------------------------------
	if (multiwatch.hold)
	{
		if (multiwatch.interval < DISP_INTERVAL) multiwatch.interval++;
		else multiwatch.interval = 0;

		if (multiwatch.interval > TEMP_INTERVAL)
		{
			digit1 = digits[multiwatch.hold];
			digit2 = cNULL; digit3 = cNULL; digit4 = cR; digit5 = cE; digit6 = cS; 
		}	
	}

	//---display data--------------------------------
	display_fill(digit1, digit2, digit3, digit4, digit5, digit6);
}


//------------------------------
void Pointer(char press)
{
	char digit1 = cNULL;
	char digit2 = cNULL;
	char digit3 = cNULL;
	char digit4 = cNULL;
	char digit5 = cNULL;
	char digit6 = cNULL;

	char temp1;
	char temp2;
	

	//---check user reaction-------------------------
	if (press)
	{
		switch (press)
		{
			case PRESSED1:
				if (pointer.count3) pointer.count3--;
				else pointer.count3 = 101;
			break;
			case PRESSED2:
				if (pointer.count3 <= 100) pointer.count3++;
				else pointer.count3 = 0;
			break;
			case PRESSED3:
				if (pointer.count2) pointer.count2--;
				else pointer.count2 = 101;
			break;
			case PRESSED4:
				if (pointer.count2 <= 100) pointer.count2++;
				else pointer.count2 = 0;
			break;				
			case PRESSED5: 
				if (pointer.count1) pointer.count1--;
				else pointer.count1 = 101;
			break;			
			case PRESSED6:
				if (pointer.count1 <= 100) pointer.count1++;
				else pointer.count1 = 0;
			break;				
		}
		press = 0;
	}
	

	//---external int-----------------------------------------
	/*if (counter.correction1 == 0)
	{
		temp2 = counter.pointer1;
		counter.correction1 = temp2;
	}
	else
		temp2 = 0;


	if (counter.correction2 == 0)
	{
		temp1 = counter.pointer2;
		counter.correction2 = temp1;
	}
	else
		temp1 = 0;
	
	if (pointer.count1 < 99)
	{	
		if (temp1 <= 99) 
		{
			pointer.count1 += temp1;
		}
		else pointer.count1 = 100;
		
		if (pointer.count1 > 100) 
			pointer.count1 = 100; 
	}	
	
	if (pointer.count3 < 99)
	{	
		if (temp2 <= 99) 
		{
			pointer.count3 += temp2;
		}
		else pointer.count3 = 100;
		
		if (pointer.count3 > 100) 
			pointer.count3 = 100; 
	}
	*/
	//---special char------------------------------
	if (pointer.count1 > 101) pointer.count1 = 0; 
	if (pointer.count1 == 101) 
	{
		digit1 = cNULL;
		digit2 = cNULL;
	}
	else if (pointer.count1 == 100)
	{
		digit1 = cMINUS;
		digit2 = cMINUS;
	}
	else if (pointer.count1 <= 99)
	{	
		digit1 = digits[(pointer.count1 / 10)];
		digit2 = digits[(pointer.count1 % 10)];	
	}


	if (pointer.count2 > 101) pointer.count2 = 0; 
	if (pointer.count2 == 101) 
	{
		digit3 = cNULL;
		digit4 = cNULL;
	}
	else if (pointer.count2 == 100)
	{
		digit3 = cMINUS;
		digit4 = cMINUS;
	}
	else if (pointer.count2 <= 99)
	{	
		digit3 = digits[(pointer.count2 / 10)];
		digit4 = digits[(pointer.count2 % 10)];	
	}


	if (pointer.count3 > 101) pointer.count3 = 0; 
	if (pointer.count3 == 101) 
	{
		digit5 = cNULL;
		digit6 = cNULL;
	}
	else if (pointer.count3 == 100)
	{
		digit5 = cMINUS;
		digit6 = cMINUS;
	}
	else if (pointer.count3 <= 99)
	{	
		digit5 = digits[(pointer.count3 / 10)];
		digit6 = digits[(pointer.count3 % 10)];	
	}

	if (pointer.count2 <= 99)
	{	
		digit2 += cDOT;
		digit4 += cDOT;
	}

	//---display data--------------------------------
	display_fill(digit1, digit2, digit3, digit4, digit5, digit6);
}


//-------------------------------------------------------------------
void Score(char press)
{
	char digit1 = cNULL;
	char digit2 = cNULL;
	char digit3 = cNULL;
	char digit4 = cNULL;
	char digit5 = cNULL;
	char digit6 = cNULL;
	char temp1;
	char temp2;
	char blank;
	uint32_t long_temp;


	//---external int-----------------------------------------
	if (counter.correction2 == 0)
	{
		temp2 = counter.pointer2;
		counter.correction2 = temp2;
	}
	else
		temp2 = 0;


	if (counter.correction1 == 0)
	{
		temp1 = counter.pointer1;
		counter.correction1 = temp1;
	}
	else
		temp1 = 0;

	if (score.mode)
	{
		if (temp1) score.counter += temp1;				//add
		if (temp2)
		{
			if (score.counter >= temp2) score.counter -= temp2;		//sub
		}
	}

	//---alarm---------------------------------------
	if (score.counter < score.terminate)
	{
		if (score.alarm == 2)	//if by wikszy a teraz nie jest
			counter.alarm = ALARM_PERIOD;	//alarm on
		score.alarm = 1;				//disable
	}
	
	if (score.counter > score.terminate)
	{
		if (score.alarm == 1)	//if by mniejszy a teraz nie jest
			counter.alarm = ALARM_PERIOD;	//alarm on
		score.alarm = 2;				//disable
	}

	if (score.counter == score.terminate)
	{
		if (score.alarm)	//if by mniejszy lub wiekszy a teraz nie jest		
			counter.alarm = ALARM_PERIOD;	//alarm on
		score.alarm = 0;				//disable
	}


	//---check user reaction-------------------------
	if (press)
	{
		//rotate digits
		if (press == PRESSED1) score.settings++;
		if (score.settings > 6) 
		{
			score.settings = 0;
			ee_32write(SCORE_EEADR, (char*) &score.terminate);
		}
		
		//change some digits 
		if (score.settings)
		{
			long_temp = 0;
			switch (score.settings)
			{
				case 1: long_temp = 100000; break;		
				case 2: long_temp = 10000; break;
				case 3: long_temp = 1000; break;
				case 4: long_temp = 100; break;
				case 5: long_temp = 10; break;
				case 6: long_temp = 1; break;
			}
			switch (press)
			{
				case PRESSED2:
					if (score.terminate >= long_temp) 
					score.terminate -= long_temp;	
				break;
				case PRESSED3:
					score.terminate += long_temp;
				break;
				case PRESSED4:
					score.terminate = 0;
				break;
				case PRESSED5:
					score.terminate = score.counter;
				break;
			}
			if (score.terminate >= 1000000) score.terminate = 999999;
		}
		else
		{	
			//other action
			switch (press)
			{
				case PRESSED3:
					score.mode = MODE_READY;
				break;
				case PRESSED4:
					score.counter = 0;
					score.mode = MODE_READY;
				break;				
				case PRESSED5: 
					score.mode = MODE_STOP;
				break;			
				case PRESSED6: 
					score.counter = score.terminate;
					score.mode = MODE_READY;
				break;			
			}
		}
		blink = 0;
		press = 0;
	}

	//---be care------------------------------------------
	if (score.counter >= 1000000) score.counter = 999999;

	//---select disp values-------------------------------
	if (score.settings) 
		long_temp = score.terminate;
	else 
		long_temp = score.counter;


	//---calculate-----------------------------------------
	digit1 = (long_temp / 100000);
	long_temp %= 100000;
	digit2 = (long_temp / 10000);
	long_temp %= 10000;
	digit3 = (long_temp / 1000);
	long_temp %= 1000;
	digit4 = (long_temp / 100);
	long_temp %= 100;
	digit5 = (long_temp / 10);
	long_temp %= 10;
	digit6 = (long_temp & 0xFF);

	blank = 1;
	if (digit1) blank = 0; 
	if (blank) digit1 = cNULL;
	else  digit1 = digits[digit1];

	if (digit2) blank = 0; 
	if (blank) digit2 = cNULL;
	else  digit2 = digits[digit2];
	
	if (digit3) blank = 0; 
	if (blank) digit3 = cNULL;
	else  digit3 = digits[digit3];

	if (digit4) blank = 0;
	if (blank) digit4 = cNULL;
	else  digit4 = digits[digit4];

	if (digit5) blank = 0; 
	if (blank) digit5 = cNULL;
	else  digit5 = digits[digit5];

	digit6 = digits[digit6];

	if (score.settings)
	{
		blink++;
		if (blink > BLINK_INTERVAL) blink = 0;
		if (blink > BLINK_PERIOD)
		{
			switch (score.settings)
			{
				case 1: digit1 = cCURSOR; break;
				case 2: digit2 = cCURSOR; break;
				case 3: digit3 = cCURSOR; break;
				case 4: digit4 = cCURSOR; break;
				case 5: digit5 = cCURSOR; break;
				case 6: digit6 = cCURSOR; break;		
			}
		}
	}
	else
	{
		if (!(score.mode)) digit6 |= cDOT;
	}

	//---display data--------------------------------
	display_fill(digit1, digit2, digit3, digit4, digit5, digit6);
}


//-------------------------------------------------------------------
void CountDown(char press)
{
	uint32_t long_temp;
	char digit1 = cNULL;
	char digit2 = cNULL;
	char digit3 = cNULL;
	char digit4 = cNULL;
	char digit5 = cNULL;
	char digit6 = cNULL;
	char dots;


	//---get score-----------------------------------
	counter.hold = COUNTER_CATCH;
	while (counter.hold);

	if (down.terminate >= counter.hold_counter)
	{
		down.counter = (down.terminate - counter.hold_counter); 
		down.alarm = 1;
	}
	else
	{	
		down.counter = 0;
		if (down.alarm) counter.alarm = ALARM_PERIOD;	//alarm on
		down.alarm = 0;				//disable		
	}


	//---check user reaction-------------------------
	if (press)
	{
		//rotate digits
		if (press == PRESSED1) down.settings++;
		if (down.settings > 3)
		{
			down.settings = 0;
			ee_32write(DOWN_EEADR, (char*) &down.terminate);
		}
		
		//change some digits 
		if (down.settings)
		{
			long_temp = 0;
			switch (down.settings)
			{
				case 1: long_temp = MSEC_ON_HOUR; 		break;
				case 2: long_temp = MSEC_ON_MIN; 		break;
				case 3: long_temp = MSEC_ON_SEC; 		break;
			}
			switch (press)
			{
				case PRESSED2:
					if (down.terminate >= long_temp) 
					down.terminate -= long_temp;	
				break;
				case PRESSED3:
					down.terminate += long_temp;
				break;
				case PRESSED4:
					down.terminate = 0;
				break;				
				case PRESSED5:
					down.terminate = down.counter;			//manual stop
				break;				
			}
			//reset msec
			long_temp = down.terminate % MSEC_ON_SEC;
			if (long_temp) down.terminate -= long_temp;
		}
		else
		{
		
			//other action
			switch (press)
			{
				case PRESSED3:
					counter.state = COUNTER_RUN;
				break;				
				case PRESSED4:
					counter.state = COUNTER_STOP;			//manual stop
					down.counter = 0;						//zero display
					counter.counter = down.terminate;
				break;				
				case PRESSED5:
					counter.state = COUNTER_STOP;			//manual stop
				break;				
				case PRESSED6: 
					counter.state = COUNTER_STOP;			//manual stop
					down.counter = down.terminate;			//reload
					counter.counter = 0;
				break;			
			}
		}

		blink = 0;
		press = 0;
	}

	//---select disp values-------------------------------
	//---be care------------------------------------------
	if (down.counter   > MSEC_ON_99HOUR) down.counter   = MSEC_ON_99HOUR;
	if (down.terminate > MSEC_ON_99HOUR) down.terminate = MSEC_ON_99HOUR;

	if (down.settings)
		long_temp = down.terminate;
	else
		long_temp = down.counter;
	
	dots = 0;
	if ((long_temp >= MSEC_ON_HOUR) || (down.settings))
	{
		digit1 = (long_temp / MSEC_ON_10HOUR);
		long_temp %= MSEC_ON_10HOUR;
		digit2 = (long_temp / MSEC_ON_HOUR);
		long_temp %= MSEC_ON_HOUR;
		digit3 = (long_temp / MSEC_ON_10MIN);
		long_temp %= MSEC_ON_10MIN;
		digit4 = (long_temp / MSEC_ON_MIN);
		long_temp %= MSEC_ON_MIN;
		digit5 = (long_temp / MSEC_ON_10SEC);
		long_temp %= MSEC_ON_10SEC;
		digit6 = (long_temp / MSEC_ON_SEC);
		long_temp %= MSEC_ON_SEC;
		if (long_temp < 50) dots = 3;
		dots |= 4;		
	}
	else
	{
		long_temp %= MSEC_ON_HOUR;
		digit1 = (long_temp / MSEC_ON_10MIN);
		long_temp %= MSEC_ON_10MIN;
		digit2 = (long_temp / MSEC_ON_MIN);
		long_temp %= MSEC_ON_MIN;
		digit3 = (long_temp / MSEC_ON_10SEC);
		long_temp %= MSEC_ON_10SEC;
		digit4 = (long_temp / MSEC_ON_SEC);	
		long_temp %= MSEC_ON_SEC;		
		digit5 = (long_temp / 10);
		long_temp %= 10;
		digit6 = (long_temp & 0xFF);
		if (digit5 < 5) 	dots |= 1;
		if (!(digit5 & 1))  dots |= 2;
	}

	if (digit1) 
		digit1 = digits[digit1];
	else 
		digit1 = cNULL;

	digit2 = digits[digit2];
	digit3 = digits[digit3];
	digit4 = digits[digit4];
	digit5 = digits[digit5];
	digit6 = digits[digit6];
	
	if (down.settings)
	{
		blink++;
		if (blink > BLINK_INTERVAL) blink = 0;
		if (blink > BLINK_PERIOD)
		{
			switch (down.settings)
			{
				case 1: digit1 = cNULL; digit2 = cNULL; break;
				case 2: digit3 = cNULL; digit4 = cNULL; break;
				case 3: digit5 = cNULL; digit6 = cNULL; break;
			}
		}
		digit2 |= cDOT;
		digit4 |= cDOT;
		if (dots & 4) digit6 |= cDOT;
	}
	else 
	{
		if (counter.state == COUNTER_STOP)
		{
			digit2 |= cDOT;
			digit4 |= cDOT;
			if (dots & 4) digit6 |= cDOT;
		}
		else
		{
			if (dots & 1) digit2 |= cDOT;
			if (dots & 2) digit4 |= cDOT;
			if (dots & 4) digit6 |= cDOT;
		}
	}	

	//---display data--------------------------------
	display_fill(digit1, digit2, digit3, digit4, digit5, digit6);
}

