/*
 * megas.c
 *
 * Created: 2012-08-02 12:43:00
 *  Author: dsosnowski
 */ 

#define F_CPU 10000000

//includes
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
//#include <avr/wdt.h>
#include <avr/sleep.h>

#include "characters.h"
#include "defs.h"
#include "func.h"
#include "wdog.h"

#define ENABLE_EINT			GICR = (1<<INT1 | 1<<INT0)
#define DISABLE_EINT		GICR = 0
#define CLEAR_EINT			GIFR = (1<<INTF1 | 1<<INTF0)					


//----------------------------------------
void hard_init(void);
void display_refresh(void);
void show_intro(void);
void slide_show(char c1, char c2, char c3, char c4, char c5, char c6);
void display_fill(char v1, char v2, char v3, char v4, char v5, char v6);
void switch_scan(void);

//--------------------   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,   0,  -
const char digits[] = {125,  68, 122, 110,  71,  47,  63, 101, 127, 111, 125,  2}; 

static volatile char display[6];
static char key_pressed = HOLDING1;

//----------------------------------
struct{
	volatile uint32_t counter;	
	volatile char hold;
	volatile char correct;
	volatile uint32_t hold_counter;
}master;

//----------------------------------
struct{
	volatile uint32_t counter;
	volatile uint32_t hold_counter;
	volatile char alarm;
	volatile char state;	
	volatile char hold;
	volatile char pointer1;
	volatile char pointer2;
	volatile char correction1;
	volatile char correction2;
}counter;

//----------------------------------
extern struct{
	uint32_t counter;
	uint32_t result1;			
	uint32_t result2;			
	uint32_t result3;			
	uint32_t result4;			
	uint32_t result5;			
	char settings;
	char result;
	char hold;
	unsigned int interval;
	char dominant;	
}multiwatch;



//----------------------------------
char function;

int main(void)
{
	hard_init();
	show_intro();
	Restore();

	while(1)
    {
		
		//--------------------------------------------------------------------
		switch_scan();
		
		//change func
		//--------------------------------------------------------------------
		if (key_pressed > HOLDING)
		{
			switch (key_pressed)
			{
				case HOLDING1: function = FUNC_CLOCK + SWITCH_FUNCTION;			break;
				case HOLDING2: function = FUNC_STOPWATCH + SWITCH_FUNCTION;		break;
				case HOLDING3: function = FUNC_MULTIWATCH + SWITCH_FUNCTION;	break;
				case HOLDING4: function = FUNC_POINTER + SWITCH_FUNCTION;		break;
				case HOLDING5: function = FUNC_POINTER2 + SWITCH_FUNCTION;		break;
				case HOLDING6: function = FUNC_CDOWN + SWITCH_FUNCTION;			break;				
			}
			key_pressed = 0;
			//multiwatch.result = 0;		//change func resets multiwatch
			multiwatch.hold = 0;
		}		
		
		//switch func info
		//--------------------------------------------------------------------
		if (function >= SWITCH_FUNCTION)
		{		
			switch (function)
			{
				case (FUNC_CLOCK + SWITCH_FUNCTION):	
					function -= SWITCH_FUNCTION;
					slide_show(cNULL, cZ, cE, cG, cA, cR);				break;
					
				case (FUNC_STOPWATCH + SWITCH_FUNCTION):	
					function -= SWITCH_FUNCTION;
					slide_show(cS, cT, cO, cP, cE, cR);					break;

				case (FUNC_MULTIWATCH + SWITCH_FUNCTION):	
					function -= SWITCH_FUNCTION;
					slide_show(cS, cT, cO, cP, cNULL, d5);				break;

				case (FUNC_POINTER + SWITCH_FUNCTION):	
					function -= SWITCH_FUNCTION;
					slide_show(cM, cM, cE, cC, cZ, cNULL);				break;
					
				case (FUNC_POINTER2 + SWITCH_FUNCTION):	
					function -= SWITCH_FUNCTION;
					slide_show(cNULL, cL, cI, cC, cZ, cNULL);			break;

				case (FUNC_CDOWN + SWITCH_FUNCTION):	
					function -= SWITCH_FUNCTION;
					slide_show(cO, cD, cL, cI, cC, cZ);					break;
			}
		}
		//func exe
		//--------------------------------------------------------------------
		switch (function)
		{
			case FUNC_CLOCK:		Clock(key_pressed);		 break;
			case FUNC_STOPWATCH:	StopWatch(key_pressed);	 break;
			case FUNC_MULTIWATCH:	MultiWatch(key_pressed); break;
			case FUNC_POINTER:		Pointer(key_pressed);	 break;
			case FUNC_POINTER2:		Score(key_pressed);	 	 break;
			case FUNC_CDOWN:		CountDown(key_pressed);	 break;					
		}	
		key_pressed = 0;
		
		
		//be care
		//--------------------------------------------------------------------
		if (function > FUNC_CDOWN) function = FUNC_CLOCK + SWITCH_FUNCTION;				
		
		//wdog
		//--------------------------
		WdtReset();

		//main delay		
		//--------------------------------------------------------------------
		_delay_ms(MAIN_DELAY);
	}	
}


//-----------------------------------------------
void switch_scan(void)
{

	static char hold_counter;
	static char key_prev;
	char key_hold;

	key_hold = SWITCH_PORT;
	key_hold |= (1<<7 | 1<<6);
	key_hold = ~(key_hold);
	
	if (key_hold)
	{
		key_prev = key_hold;
		if (hold_counter <= SW_HOLD_TIME) hold_counter++;
		if (hold_counter == SW_HOLD_TIME)
		{
			key_pressed = (key_hold + HOLDING);
			hold_counter -= SW_HOLD_BACK;
		}		
	}
	else
	{	if (hold_counter)
		{
			if (hold_counter < SW_HOLD_TIME)
				key_pressed = key_prev;
		}
		hold_counter = 0;
	}
}

//-------------------------
void show_intro(void)
{
	slide_show(cNULL, cA, cV, cT, cNULL, cNULL);
	slide_show(cNULL, d5, d3, d9, d9, cNULL);
}

//-------------------------
void slide_show(char c1, char c2, char c3, char c4, char c5, char c6)
{	
	display[0] = cNULL;
	display[1] = cNULL;
	display[2] = cNULL;
	display[3] = cNULL;
	display[4] = cNULL;
	display[5] = cNULL;
	_delay_ms(SLIDE_SPEED);
	WdtReset();

	display[0] = c1;
	_delay_ms(SLIDE_SPEED);

	display[1] = c2;
	_delay_ms(SLIDE_SPEED);
	WdtReset();

	display[2] = c3;
	_delay_ms(SLIDE_SPEED);

	display[3] = c4;
	_delay_ms(SLIDE_SPEED);
	WdtReset();

	display[4] = c5;
	_delay_ms(SLIDE_SPEED);

	display[5] = c6;
	_delay_ms(SLIDE_SPEED);
	_delay_ms(400);
	WdtReset();
}

//-------------------------
void display_fill(char v1, char v2, char v3, char v4, char v5, char v6)
{
	display[0] = v1;
	display[1] = v2;
	display[2] = v3;
	display[3] = v4;
	display[4] = v5;
	display[5] = v6;		
}

//-------------------------
void display_refresh(void)
{
	static char anode;
		
	if (anode < 6) anode++;
	else anode = 0;
	
	ANODES_PORT &= ~(0x3F);

	switch (anode)
	{
		case 0x00:
			DISP_PORT = display[0];
			ANODES_PORT |= 0x01;	
		break;
		case 0x01:
			DISP_PORT = display[1];
			ANODES_PORT |= 0x02;	
		break;
		case 0x02:
			DISP_PORT = display[2];
			ANODES_PORT |= 0x04;	
		break;
		case 0x03:
			DISP_PORT = display[3];
			ANODES_PORT |= 0x08;	
		break;
		case 0x04:
			DISP_PORT = display[4];
			ANODES_PORT |= 0x10;	
		break;
		case 0x05:
			DISP_PORT = display[5];
			ANODES_PORT |= 0x20;	
		break;
		case 0x06:
			DISP_PORT = 0x00;
		break;			
	}
}

//-------------------------
void hard_init(void)
{
	ANODES_PORT = 0;
	ANODES_DDR = (1<<ANODE_1 | 1<<ANODE_2 | 1<<ANODE_3 | 1<<ANODE_4 | 1<<ANODE_5 | 1<<ANODE_6);

	DISP_PORT = 0;
	DISP_DDR = MAX;

	SWITCH_DDR = 0;
	
	EXT_PORT = (1<<FOTO1 | 1<<FOTO2);
	EXT_DDR = (1<<ALARM);
	
	MCUCR = (1<<ISC11 | 1<<ISC01);
	ENABLE_EINT;

	TCCR0 = (1<<WGM01 | 1<<CS02);
	OCR0 = DISP_REFRESH_FREQ;
	
	TCCR1A = (1<<WGM11);
	TCCR1B = (1<<WGM12 | 1<<WGM13 | 1<<CS11);
	ICR1 = 12500;
	
	TIMSK = (1<<OCIE0 | 1<<TOIE1);
	
	WdtSet(7);

	sei();
}

//-----------------------------------
SIGNAL(TIMER0_COMP_vect)
{
	display_refresh();
}

//-----------------------------------
SIGNAL(TIMER1_OVF_vect)
{
	master.counter++;

	if (master.correct) 
	{
		master.counter -= MSEC_ON_DAY;
		master.correct = DISABLE;
	}		
	if (master.hold == MASTER_CHANGE) master.counter = master.hold_counter;
	if (master.hold == MASTER_CATCH) master.hold_counter = master.counter;	
	if (master.hold) master.hold = DISABLE;		
	
	if (counter.state) counter.counter++;
	
	if (counter.alarm)
	{
		ALARM_SET;
		counter.alarm--;
	}
	else ALARM_CLR; 
	
	if (counter.hold)
	{
		counter.hold_counter = counter.counter;
		counter.hold = DISABLE;
	}
}

//------------------------------------
SIGNAL(INT0_vect)
{
	counter.state = COUNTER_STOP;
	counter.hold = COUNTER_CATCH;						//get counter state
	counter.pointer2++;
	if (counter.pointer2 > counter.correction2) 
		counter.pointer2 -= counter.correction2;
	counter.correction2 = 0;

	if (multiwatch.dominant)
	{
		if (multiwatch.result)
		{
			counter.state = COUNTER_RUN;
			switch (multiwatch.result)
			{
				case 5: multiwatch.result1 = counter.counter; break;
				case 4: multiwatch.result2 = counter.counter; break;
				case 3: multiwatch.result3 = counter.counter; break;
				case 2: multiwatch.result4 = counter.counter; break;
				case 1: multiwatch.result5 = counter.counter; break;
			}
			multiwatch.result--;
		}
		if (!(multiwatch.result)) counter.state = COUNTER_STOP;	
	}
	else multiwatch.result = 0;	
}

//------------------------------------
SIGNAL(INT1_vect)
{
	counter.state = COUNTER_RUN;
	counter.pointer1++;
	if (counter.pointer1 > counter.correction1) 
		counter.pointer1 -= counter.correction1;
	counter.correction1 = 0;
	if (!(multiwatch.result)) multiwatch.result = 5;	
}
